@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -106,16 +106,12 @@ static void at_csca_set(struct ofono_sms *sms,
 	struct cb_data *cbd = cb_data_new(cb, user_data);
 	char buf[64];
 
-	if (cbd == NULL)
-		goto error;
-
 	snprintf(buf, sizeof(buf), "AT+CSCA=\"%s\",%d", sca->number, sca->type);
 
 	if (g_at_chat_send(data->chat, buf, csca_prefix,
 				at_csca_set_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, user_data);
@@ -173,14 +169,10 @@ static void at_csca_query(struct ofono_sms *sms, ofono_sms_sca_query_cb_t cb,
 	struct sms_data *data = ofono_sms_get_data(sms);
 	struct cb_data *cbd = cb_data_new(cb, user_data);
 
-	if (cbd == NULL)
-		goto error;
-
 	if (g_at_chat_send(data->chat, "AT+CSCA?", csca_prefix,
 				at_csca_query_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, NULL, user_data);
@@ -227,9 +219,6 @@ static void at_cmgs(struct ofono_sms *sms, unsigned char *pdu, int pdu_len,
 	char buf[512];
 	int len;
 
-	if (cbd == NULL)
-		goto error;
-
 	if (mms) {
 		snprintf(buf, sizeof(buf), "AT+CMMS=%d", mms);
 		g_at_chat_send(data->chat, buf, none_prefix,
@@ -243,7 +232,6 @@ static void at_cmgs(struct ofono_sms *sms, unsigned char *pdu, int pdu_len,
 				at_cmgs_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, -1, user_data);
@@ -267,16 +255,12 @@ static void at_cgsms_set(struct ofono_sms *sms, int bearer,
 	struct cb_data *cbd = cb_data_new(cb, user_data);
 	char buf[64];
 
-	if (cbd == NULL)
-		goto error;
-
 	snprintf(buf, sizeof(buf), "AT+CGSMS=%d", bearer);
 
 	if (g_at_chat_send(data->chat, buf, none_prefix,
 				at_cgsms_set_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, user_data);
@@ -319,14 +303,10 @@ static void at_cgsms_query(struct ofono_sms *sms,
 	struct sms_data *data = ofono_sms_get_data(sms);
 	struct cb_data *cbd = cb_data_new(cb, user_data);
 
-	if (cbd == NULL)
-		goto error;
-
 	if (g_at_chat_send(data->chat, "AT+CGSMS?", cgsms_prefix,
 				at_cgsms_query_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, -1, user_data);
@@ -339,17 +319,16 @@ static void at_cnma_cb(gboolean ok, GAtResult *result, gpointer user_data)
 				"Further SMS reception is not guaranteed");
 }
 
-static gboolean at_parse_pdu_common(GAtResult *result, const char *prefix,
-					const char **pdu, int *pdulen)
+static gboolean at_parse_cmt(GAtResult *result,	const char **pdu, int *pdulen)
 {
 	GAtResultIter iter;
 
 	g_at_result_iter_init(&iter, result);
 
-	if (!g_at_result_iter_next(&iter, prefix))
+	if (!g_at_result_iter_next(&iter, "+CMT:"))
 		return FALSE;
 
-	if (!strcmp(prefix, "+CMT:") && !g_at_result_iter_skip_next(&iter))
+	if (!g_at_result_iter_skip_next(&iter))
 		return FALSE;
 
 	if (!g_at_result_iter_next_number(&iter, pdulen))
@@ -385,11 +364,26 @@ static void at_cds_notify(GAtResult *result, gpointer user_data)
 	int tpdu_len;
 	const char *hexpdu;
 	unsigned char pdu[176];
+	GAtResultIter iter;
 
-	if (!at_parse_pdu_common(result, "+CDS:", &hexpdu, &tpdu_len)) {
-		ofono_error("Unable to parse CDS notification");
-		return;
-	}
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+CDS:"))
+		goto err;
+
+	/*
+	 * Quirk for ZTE firmware which is not compliant with 27.005
+	 * The +CDS syntax used by ZTE is including a comma before the length
+	 * +CDS: ,<length><CR><LF><pdu>
+	 * As a result, we need to skip this omitted subparameter
+	 */
+	if (data->vendor == OFONO_VENDOR_ZTE)
+		g_at_result_iter_skip_next(&iter);
+
+	if (!g_at_result_iter_next_number(&iter, &tpdu_len))
+		goto err;
+
+	hexpdu = g_at_result_pdu(result);
 
 	if (strlen(hexpdu) > sizeof(pdu) * 2) {
 		ofono_error("Bad PDU length in CDS notification");
@@ -404,17 +398,23 @@ static void at_cds_notify(GAtResult *result, gpointer user_data)
 
 	if (data->cnma_enabled)
 		at_ack_delivery(sms);
+
+	return;
+
+err:
+	ofono_error("Unable to parse CDS notification");
 }
 
 static void at_cmt_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_sms *sms = user_data;
+	struct sms_data *data = ofono_sms_get_data(sms);
 	const char *hexpdu;
 	long pdu_len;
 	int tpdu_len;
 	unsigned char pdu[176];
 
-	if (!at_parse_pdu_common(result, "+CMT:", &hexpdu, &tpdu_len)) {
+	if (!at_parse_cmt(result, &hexpdu, &tpdu_len)) {
 		ofono_error("Unable to parse CMT notification");
 		return;
 	}
@@ -429,7 +429,8 @@ static void at_cmt_notify(GAtResult *result, gpointer user_data)
 	decode_hex_own_buf(hexpdu, -1, &pdu_len, 0, pdu);
 	ofono_sms_deliver_notify(sms, pdu, pdu_len, tpdu_len);
 
-	at_ack_delivery(sms);
+	if (data->vendor != OFONO_VENDOR_SIMCOM)
+		at_ack_delivery(sms);
 }
 
 static void at_cmgr_notify(GAtResult *result, gpointer user_data)
@@ -798,15 +799,21 @@ static gboolean build_cnmi_string(char *buf, int *cnmi_opts,
 
 	DBG("");
 
-	if (data->vendor == OFONO_VENDOR_QUALCOMM_MSM ||
-			data->vendor == OFONO_VENDOR_HUAWEI ||
-			data->vendor == OFONO_VENDOR_NOVATEL)
+	switch (data->vendor) {
+	case OFONO_VENDOR_GOBI:
+	case OFONO_VENDOR_QUALCOMM_MSM:
+	case OFONO_VENDOR_NOVATEL:
+	case OFONO_VENDOR_HUAWEI:
+	case OFONO_VENDOR_ZTE:
 		/* MSM devices advertise support for mode 2, but return an
 		 * error if we attempt to actually use it. */
 		mode = "1";
-	else
+		break;
+	default:
 		/* Sounds like 2 is the sanest mode */
 		mode = "2310";
+		break;
+	}
 
 	if (!append_cnmi_element(buf, &len, cnmi_opts[0], mode, FALSE))
 		return FALSE;
@@ -826,10 +833,14 @@ static gboolean build_cnmi_string(char *buf, int *cnmi_opts,
 	 * ack it with error "CNMA not expected."  However, not acking it
 	 * sends the device into la-la land.
 	 */
-	if (data->vendor == OFONO_VENDOR_NOVATEL)
+	switch (data->vendor) {
+	case OFONO_VENDOR_NOVATEL:
 		mode = "20";
-	else
+		break;
+	default:
 		mode = "120";
+		break;
+	}
 
 	/*
 	 * Try to deliver Status-Reports via +CDS, then CDSI or don't
@@ -916,11 +927,18 @@ static void at_cnmi_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		supported = TRUE;
 
 	/* support for ack pdu is not working */
-	if (data->vendor == OFONO_VENDOR_IFX ||
-			data->vendor == OFONO_VENDOR_HUAWEI ||
-			data->vendor == OFONO_VENDOR_NOVATEL ||
-			data->vendor == OFONO_VENDOR_OPTION_HSO)
+	switch (data->vendor) {
+	case OFONO_VENDOR_IFX:
+	case OFONO_VENDOR_GOBI:
+	case OFONO_VENDOR_ZTE:
+	case OFONO_VENDOR_ICERA:
+	case OFONO_VENDOR_HUAWEI:
+	case OFONO_VENDOR_NOVATEL:
+	case OFONO_VENDOR_OPTION_HSO:
 		goto out;
+	default:
+		break;
+	}
 
 	if (data->cnma_enabled)
 		construct_ack_pdu(data);
@@ -967,8 +985,11 @@ static gboolean set_cpms(gpointer user_data)
 	const char *incoming = storages[data->incoming];
 	char buf[128];
 
-	snprintf(buf, sizeof(buf), "AT+CPMS=\"%s\",\"%s\",\"%s\"",
-			store, store, incoming);
+	if (data->vendor == OFONO_VENDOR_WAVECOM_Q2XXX)
+		snprintf(buf, sizeof(buf), "AT+CPMS=\"%s\"", store);
+	else
+		snprintf(buf, sizeof(buf), "AT+CPMS=\"%s\",\"%s\",\"%s\"",
+				store, store, incoming);
 
 	g_at_chat_send(data->chat, buf, cpms_prefix,
 			at_cpms_set_cb, sms, NULL);
@@ -1020,7 +1041,7 @@ static void at_cpms_query_cb(gboolean ok, GAtResult *result,
 	gboolean supported = FALSE;
 
 	if (ok) {
-		int mem = 0;
+		int mem = 0, mem_max;
 		GAtResultIter iter;
 		const char *store;
 		gboolean me_supported[3];
@@ -1036,7 +1057,20 @@ static void at_cpms_query_cb(gboolean ok, GAtResult *result,
 		if (!g_at_result_iter_next(&iter, "+CPMS:"))
 			goto out;
 
-		for (mem = 0; mem < 3; mem++) {
+		if (data->vendor == OFONO_VENDOR_WAVECOM_Q2XXX) {
+			/* skip initial `(' */
+			if (!g_at_result_iter_open_list(&iter))
+				goto out;
+
+			/*
+			 * Wavecom Q2 replies: +CPMS: (("SM","BM","SR"),("SM"))
+			 * This reply is broken according to 3GPP TS 07.05.
+			 */
+			mem_max = 2;
+		} else
+			mem_max = 3;
+
+		for (mem = 0; mem < mem_max; mem++) {
 			if (!g_at_result_iter_open_list(&iter))
 				goto out;
 
@@ -1053,7 +1087,9 @@ static void at_cpms_query_cb(gboolean ok, GAtResult *result,
 				goto out;
 		}
 
-		if (!sm_supported[2] && !me_supported[2] && !mt_supported[2])
+		if (data->vendor != OFONO_VENDOR_WAVECOM_Q2XXX &&
+				!sm_supported[2] && !me_supported[2]
+				&& !mt_supported[2])
 			goto out;
 
 		if (sm_supported[0] && sm_supported[1]) {
@@ -1138,13 +1174,16 @@ static void at_csms_status_cb(gboolean ok, GAtResult *result,
 			goto out;
 
 
-		if (data->vendor == OFONO_VENDOR_HUAWEI ||
-				data->vendor == OFONO_VENDOR_NOVATEL) {
+		switch (data->vendor) {
+		case OFONO_VENDOR_HUAWEI:
+		case OFONO_VENDOR_NOVATEL:
 			g_at_result_iter_skip_next(&iter);
 			service = 0;
-		} else {
+			break;
+		default:
 			if (!g_at_result_iter_next_number(&iter, &service))
 				goto out;
+			break;
 		}
 
 		if (!g_at_result_iter_next_number(&iter, &mt))
@@ -1256,12 +1295,12 @@ static struct ofono_sms_driver driver = {
 	.bearer_set	= at_cgsms_set,
 };
 
-void at_sms_init()
+void at_sms_init(void)
 {
 	ofono_sms_driver_register(&driver);
 }
 
-void at_sms_exit()
+void at_sms_exit(void)
 {
 	ofono_sms_driver_unregister(&driver);
 }
